package uniba.core;

import java.util.*;
import java.io.*;

import java.awt.Frame;
import uniba.databanker.*;
import java.sql.SQLException;

/**
 * Questa classe contiene i metodi necessari all'estrazione delle fetaure. Precisamente
 * le feature si possono estrarre con tre metodi diversi:<ul><li>MaxTF-PF<sup>2</sup>-ICF</li>
 * <li>Mutual Information</li><li>Odds Ratio</li></ul>.<br>Poich il database non preserva
 * l'ordine delle feature, queste ultime vengono ordinate alfabeticamente in modo tale
 * che ogni feature occupi sempre la stessa posizione nello spazio vettoriale. 
 */
public class FeaturesManager {
   
  private CategoriesManager    cgm;
  private TrainingPagesManager trpm;
  private Vector features;
  private Vector categoryDictionaries = new Vector();
  private Vector[] dictionariesOfCategories;
  private boolean featuresExtracted = false;
  private boolean dictionariesExtracted = false;
  private boolean stopwordsEnabled = true;
  private boolean stemmingEnabled = true;
  private int     minLengthTreshold;
  private int     maxLengthTreshold;
  private int     totalFeatures;
  private int     categoryDictionaryMaxSize;
  private int     maxWordsPerCategoryAsFeatures;
  private Databanker db = null;
  private String utente = null;
  

  /**
   * Istanzia le variabili necessarie all'estrazione delle feature. Se i parametri
   * di configurazione cambiano per far funzionare bene <b>FeaturesManager</b>
   * invocare il metodo <b>updateConfiguration</b>.
   */
  public FeaturesManager(CategoriesManager cgm, TrainingPagesManager trpm,
    Databanker db, String utente) throws SQLException {
    this.cgm  = cgm;
    this.trpm = trpm;
    this.db = db;
    this.utente = utente;
    features = new Vector();
    
    Vector configuration = db.getConfiguration(utente);
    categoryDictionaryMaxSize = ((Integer)configuration.elementAt(5)).intValue();
    maxWordsPerCategoryAsFeatures = ((Integer)configuration.elementAt(6)).intValue();
    minLengthTreshold = ((Integer)configuration.elementAt(0)).intValue();
    maxLengthTreshold = ((Integer)configuration.elementAt(1)).intValue();
    features = db.getFeature(utente);
    totalFeatures = features.size();
  }

  /**
   * Da richiamare quando cambiano i parametri di configurazione.
   */
  public void updateConfiguration() throws SQLException {
      Vector configuration = db.getConfiguration(utente);
      categoryDictionaryMaxSize = ((Integer)configuration.elementAt(5)).intValue();
      maxWordsPerCategoryAsFeatures = ((Integer)configuration.elementAt(6)).intValue();
      minLengthTreshold = ((Integer)configuration.elementAt(0)).intValue();
      maxLengthTreshold = ((Integer)configuration.elementAt(1)).intValue();
  }

  /**
   * Da richiamare quando l'utente cancella le feature attraverso
   * l'interfaccia
   */
  public void removeFeature() {
      features.removeAllElements();
      totalFeatures = features.size();
  }
     

  /** 
   * Aggiunge un parola al vocabolario
   */
  public void addWordToDictionary(Hashtable dict, HtmlWord word) {
    
       dict.put(word.getText(), new Double(word.getOcc()) );
    
  }


 /* 
  * Aggiunge un parola al vocabolario della categoria se la parola non esiste 
  * altrimenti ne incrementa il numero di occorrenze 
  */
  private void addToWordsInCategory(Hashtable[] V, String word, String category){

     int currentCategory = 0;
     currentCategory = cgm.categoryIndex(category);  
     Hashtable dict = (Hashtable) V[currentCategory];
     Integer  n = (Integer)dict.get(word);
     if (n != null) {
		dict.put(word, new Integer(n.intValue() + 1));
     }
	else {
		dict.put(word, new Integer(1));	
     }
     
         
     V[currentCategory] = dict;
  }

  /* 
   * Calcola la frequenza di una parola "w" fra i vocabolari di categoria 
   */
  private double CategoryFrequency(String word, Vector categoryDictionaries) {
  int n=0;
  boolean trovato;
  for (int i=0; i<categoryDictionaries.size(); i++) {
    Vector categoryDictionary = (Vector) categoryDictionaries.elementAt(i);
    trovato=false;
    for (int j=0; (j<categoryDictionary.size()) && (!trovato); j++) {
     DoubleWeightedWord ww = (DoubleWeightedWord) categoryDictionary.elementAt(j);
     if (ww.getWord().equals(word)) {
       trovato=true;
       n++;
     }  
    }
  }
  return (double)(n)/(double)(categoryDictionaries.size());
  }


  /* 
   * Converte le occorenze di una parola all'interno di una pagina
   * in frequenze relative 
   */
  public void convertOccurenceToFrequency(Hashtable dict, int totWords) {
  
  String word;
  int freq;
  Enumeration e = dict.keys();
  
  while (e.hasMoreElements()) {
    word = (String) e.nextElement();
    Double n = (Double) dict.get(word);
    freq=(10000*n.intValue())/totWords;
    dict.put(word,new Integer(freq));
    // System.out.print(word+":"+(float)(freq)/(float)(10000)+"; ");
  }
  System.out.println();
  }

 


/* 
 * Converte le occorenze di una parola all'interno di una pagina
 * nella misura Odds Ratio
 */
 private Hashtable[] convertOccurenceToOR(Hashtable[] V){
    int totPageInCategory[] = new int[cgm.totalCategories()];
	 Hashtable dictionariesOrOfCategories[] = new Hashtable[cgm.totalCategories()];
	 
	 for (int k=0;k<cgm.totalCategories();k++){
	   dictionariesOrOfCategories[k] = new Hashtable();
	 }

	 /* Per ogni parola "k" in ogni vocabolario di pagina  si calcola
	      un nuovo peso con i seguenti dati:
			- n  il n.ro totale delle pagine di addestramento
			- nr  il n.ro totale di pagine rilevanti per la cetegoria
			- dk  il n.ro di pagine in cui k appare
   	   - rk  il n.ro di pagine rileventi per la categoria 
			     in cui k appare
			- ik  il numero di pagine non rilevanti per la categoria 
			     in cui k appare ( ik = dk - rk)
			- ni  il n.ro di pagine non rilevati per la categoria 
			     (ni = n - nr)
	*/ 
	 int n = trpm.getNumber();
	 totPageInCategory = trpm.getTotPageInCategory();
	 for (int i=0;i<cgm.totalCategories();i++){
	  Hashtable dictionaryOrOfCategory = new Hashtable();
	  Hashtable categoryDictionary = (Hashtable) V[i];
	  int nr = totPageInCategory[i];
	  Enumeration e = categoryDictionary.keys();
	  while (e.hasMoreElements()){
	    String kword = (String) e.nextElement();
		 Integer occ = (Integer) categoryDictionary.get(kword);
		 int rk = occ.intValue();
		 int dk =0;
		 for (int j=0;j<cgm.totalCategories();j++){
		   Hashtable tempCategoryDictionary = (Hashtable) V[j];
			Integer occs = (Integer) tempCategoryDictionary.get(kword); 
			if (occs != null){
			  dk += occs.intValue();
			}
		 }
		 int ik = dk-rk;
		 int ni = n-nr;
		 double pk = 0;
		 if (rk != nr){
			  pk = ((double)(rk)/(double)(nr));
			  
		 }else{
		    int nn = n*n;
			 double f = ((double)(1)/(double)(nn));
		    pk = (1-(double)(f));
		 }
		 double uk = 0;
		 if ((ik!=0)&&(ik!=ni)){
          uk = ((double)(ik)/(double)(ni));
		 }else{
		    int nn = n*n;
		    if (ik == 0){
			     uk = (1/(double)(nn));
			 }else{
			     double f = ((double)(1)/(double)(nn));
			     uk = (1-(double)(f));
			 }
		 }
       double oddskr = ((double)(pk)/(1-(double)(pk)));
		 double oddsknr = ((double)(uk)/(1-(double)(uk)));
		 double r = ((double)(oddskr)/(double)(oddsknr));
		 double ork = ((double)Math.log(r));
	    dictionaryOrOfCategory.put(kword,new Double(ork));
	  }
	  dictionariesOrOfCategories[i] = dictionaryOrOfCategory;
	 }
	 return dictionariesOrOfCategories;
  }


   /*
    * Calcola la frequenza di una parola "w" fra le pagine di 
    * una categoria: il DF(w) 
    */
  private double DF(String word, Vector pageDictionaries) {
    double n = 0;
    for (int i=0; i<pageDictionaries.size(); i++) {
      Hashtable pageDictionary = (Hashtable) pageDictionaries.elementAt(i);
      if (pageDictionary.containsKey(word)) n++;
    }
    return (double)(n)/(double)(pageDictionaries.size());
  }


  /**
   * Estrae le feature secondo il metodo <b>MaxTF-PF<sup>2</sup>-ICF</b>.<br> <b>extract</b> richiede che sia stato
   * invocato prima il metodo <b>extractDictionaries</b>.
   */
  public void extract(Frame progressWindow) throws SQLException, OperationException {
  String spazi="                           ";
  /* elenca tutti i vocabolari di categoria */
  for (int i=0; i<categoryDictionaries.size(); i++) {
    /* per ogni vocabolario di categoria ricalcola il peso delle singole
     parole in base al TF-IDF */
    Vector categoryDictionary = (Vector) categoryDictionaries.elementAt(i);


    for (int j=0; j<categoryDictionary.size(); j++) {
    DoubleWeightedWord ww = (DoubleWeightedWord) categoryDictionary.elementAt(j);
    String word = ww.getWord();
    double tf=((double)ww.getWeight());
    double df=CategoryFrequency(word,categoryDictionaries);
    double tfidf = (tf*(double)Math.pow(1/df,2));
    DoubleWeightedWord ww2 = new DoubleWeightedWord(word,tfidf);
    categoryDictionary.setElementAt(ww2,j);  
    if (word.length()>20) {word=word.substring(1,20);}
    }
    
    /* Riordina il vocabolario in base ai nuovi pesi appena calcolati.
    Oss: il vocabolario  gi parzialmente ordinato a causa del limitato 
    numero di parole il cui peso iniziale viene modificato dal TF-IDF */
    
    /* Bubble-sort. Ordine decrescente  */ 
    DoubleWeightedWord ww1;
    DoubleWeightedWord ww2;
    boolean scambi=true;;
    for (int j=0; ((j<categoryDictionary.size()-1) && scambi); j++) {  
    scambi=false;
    for (int k=j+1; k<categoryDictionary.size(); k++) {
      ww1  = (DoubleWeightedWord) categoryDictionary.elementAt(k-1);
      ww2 = (DoubleWeightedWord) categoryDictionary.elementAt(k);
      if (ww1.getWeight()<ww2.getWeight()) {
       categoryDictionary.setElementAt(ww2,k-1);
       categoryDictionary.setElementAt(ww1,k);
       scambi=true;
      }
    }  
    }
     if (progressWindow != null)	
     		progressWindow.hide();

  }
  /* crea il vettore delle features con le parole dei vocabolari di categoria 
     pesate con la media dei TF-IDF */
  features.removeAllElements();
  String word;
  for (int i=0; i < cgm.totalCategories(); i++) {  
    Vector categoryDictionary = (Vector) categoryDictionaries.elementAt(i);
    String categoria = cgm.categoryName(i);
    int n=0;
    for (int j=0; (j<categoryDictionary.size() && n < maxWordsPerCategoryAsFeatures); j++) {
         DoubleWeightedWord ww = (DoubleWeightedWord) categoryDictionary.elementAt(j);
         boolean presente = false; 
         for (int k = 0; k < features.size() && !presente; k++) {
               Feature f = (Feature)features.elementAt(k);
               String sf = f.getFeature();	
    		   if (ww.getWord().equals(sf)) {
                      presente = true;
               }
	   }
	   if (!presente) {
		Feature w = new Feature( (String)ww.getWord(), categoria, ww.getWeight() );
            features.addElement(w);
            n++;
	   }
    }  
  }
  db.putFeature(utente, features, "ef1", db.MAXTF);
  features.removeAllElements();
  //Ricarica le feature in ordine alfabetico
  features = db.getFeature(utente);
  totalFeatures = features.size();
  featuresExtracted=true;
  }




  /*
   * Estrae il dizionario di categoria per <i>category</i>.
   * Questo dizionario sar usato dal metodo MaxTF-PF<sup>2</sup>-ICF. 
   */
  private Vector extractCategoryDictionary(String category)
    throws SQLException {
	Vector result = new Vector();
	result = db.dictionary(utente, category);
	return result;
  }   // end extractCategoryDictionary

  

  /**
   * Estrae il dizionario che sar usato dal metodo MaxTF-PF<sup>2</sup>-ICF
   */
  public void extractDictionaries(Frame progressWindow)
    throws SQLException {
  Enumeration ecat;
  String category;
  categoryDictionaries.removeAllElements();
  /* elenca tutte le categorie */
  ecat = cgm.listTrainingCategories();
  while (ecat.hasMoreElements()) {
    category = (String) ecat.nextElement();
    categoryDictionaries.addElement(extractCategoryDictionary(category));
  }   
  dictionariesExtracted = true;
  System.out.println("Dictionaries extracted.");
  if (progressWindow != null)	
  	progressWindow.hide();
  }



  /**
   * Estrae le feature con il metodo della <b>Average Mutual Information</b>.
   */
 public void extractFeaturesMI(Frame progressWindow) throws SQLException, OperationException  {

	int totCat = cgm.totalCategories();
	int totPages = trpm.getNumber();
	Hashtable wordsInCategories[] = extractWordsInCategories();
	Hashtable temp[] = new Hashtable[totCat];
	Vector paroleTrovate = new Vector();
					
	for (int i=0; i<totCat; i++) {
		Hashtable dizionarioCorrente = (Hashtable)wordsInCategories[i];
            Hashtable temp2 = new Hashtable();
		Enumeration k = dizionarioCorrente.keys();
		System.out.println(cgm.categoryName(i));
		while (k.hasMoreElements()) {
			String w = (String)k.nextElement();
			if (!paroleTrovate.contains(w)) {
				paroleTrovate.addElement(w);
				int num = pParolaPresente(w, wordsInCategories);
				double pPresente = (double)num/(double)totPages;
				double pAssente = (double)(totPages - num)/(double)totPages;
				double mi = 0;
				if (pAssente != 0) {
					double sum1 = 0;
					double sum2 = 0;
					double c1;
					double c2;
					for (int j=0; j<totCat; j++) {
						c1 = pCategoriaParolaPresente(cgm.categoryName(j), w, 							wordsInCategories, num);
						c2 = pCategoriaParolaAssente(cgm.categoryName(j), w, 								wordsInCategories, num);
						if (c1 != 0)
							sum1 = sum1 + c1*Math.log(c1);
						if (c2 != 0)
							sum2 = sum2 + c2*Math.log(c2);
					}
					//Si assume l'equiprobabilit delle categorie
					mi = - Math.log((double)1/(double)totCat) + pPresente*sum1 + 						pAssente*sum2;
				} 
				temp2.put(w, new Double(mi));
			}
		}
		temp[i] = (temp2);
	}
	features.removeAllElements();
	Vector[] v = sortDictionaries(temp);
	for (int i=0; i<v.length; i++) {
		Vector c = (Vector)v[i];
		for (int j=0; j<c.size(); j++) {
			DoubleWeightedWord dw = (DoubleWeightedWord)c.elementAt(j);
			Feature w = new Feature( dw.getWord(), cgm.categoryName(i), dw.getWeight() );
			features.addElement( w );
		}
	}
	db.putFeature(utente, features, "ef2", db.AMI);
      features.removeAllElements();
      //Ricarica le feature in ordine alfabetico
      features = db.getFeature(utente);
      totalFeatures = features.size();
      featuresExtracted = true;
      System.out.println("Features extracted");
	if (progressWindow != null)
		progressWindow.hide();
      

}

/**
 * Calcola la probabilit che la parola <code>w</code> sia presente
 */
private int pParolaPresente(String w, Hashtable [] V) {
	int totCat = cgm.totalCategories();
	int occ = 0;
	for (int j=0; j<totCat; j++) {
		Hashtable dizionarioCorrente = (Hashtable)V[j];
		Integer n = (Integer)dizionarioCorrente.get(w);
		if (n != null)
			occ = occ + n.intValue();
	}
	return occ;
}

/**
 * Calcola la probabilit condizionata di <code>categoria</code> nota la presenza
 * di <code>parola</code>
 */
private double pCategoriaParolaPresente(String categoria, String parola, Hashtable[] V, int num) {
	Hashtable dizionarioCorrente = (Hashtable)V[cgm.categoryIndex(categoria)];
	Integer n = (Integer)dizionarioCorrente.get(parola);
	int occ = 0;
	if (n != null)
		occ = n.intValue();
	return ((double)occ/(double)num);
}

/**
 * Calcola la probabilit condizionata di <code>categoria</code> nota l'assenza
 * di <code>parola</code>
 */
private double pCategoriaParolaAssente(String categoria, String parola, Hashtable[] V, int num) {
	Hashtable dizionarioCorrente = (Hashtable)V[cgm.categoryIndex(categoria)];
	Integer n = (Integer)dizionarioCorrente.get(parola);
	int numPage[] = trpm.getTotPageInCategory();
	int occ = 0;
	if (n != null)
		occ = n.intValue();
	return ((double)(numPage[cgm.categoryIndex(categoria)]-occ)/(double)(trpm.getNumber()-num));
}

  /**
   * Estrare le feature con il metodo <b>Odds Ratio</b>.
   */
  public void extractFeaturesOR(Frame progressWindow)
    throws SQLException, OperationException {
    Hashtable wordsInCategories[] = new Hashtable[cgm.totalCategories()];
	 Hashtable wordsORInCategories[] = new Hashtable[cgm.totalCategories()]; 

	 wordsInCategories = extractWordsInCategories();
	 wordsORInCategories = convertOccurenceToOR(wordsInCategories);
	 dictionariesOfCategories = sortDictionaries(wordsORInCategories);
	 /*crea il vettore delle features con le parole dei vocabolari di 
	   categoria pesate con OddsRatio */
    features.removeAllElements();
	 for(int i=0;i<cgm.totalCategories();i++){
	   Vector category = (Vector) dictionariesOfCategories[i];
		int n = 0;
		for(int j=0;(j<category.size()&&n<maxWordsPerCategoryAsFeatures);j++){
		   DoubleWeightedWord dw = (DoubleWeightedWord)category.elementAt(j);
			String word = (String) dw.getWord();
			boolean presente = false; 
         		for (int k = 0; k < features.size() && !presente; k++) {
               		Feature f = (Feature)features.elementAt(k);
               		String sf = f.getFeature();	
    		   		if (dw.getWord().equals(sf)) {
                      		presente = true;
               		}
	   		}
	   		if (!presente) {
				Feature w = new Feature( (String)dw.getWord(), cgm.categoryName(i),dw.getWeight() );
            		features.addElement(w);
            		n++;
	   		}
		}
	 }
	 
        
      db.putFeature(utente, features, "ef3", db.ODDS);
      features.removeAllElements();
      //Ricarica le feature in ordine alfabetico
      features = db.getFeature(utente);
      totalFeatures = features.size();
      featuresExtracted = true;
   System.out.println("Features extracted");
   System.err.println("Features extracted");
   if (progressWindow != null)	
   	progressWindow.hide();
  }
 
  /*
   */
  private Hashtable[] extractWordsInCategories() throws SQLException {
    	
	 Hashtable dictionariesOfCategories[] = new Hashtable[cgm.totalCategories()];
	 for (int i=0; i<cgm.totalCategories(); i++)
		dictionariesOfCategories[i] = db.getWordsInCategory(utente, cgm.categoryName(i));
	 return dictionariesOfCategories;
  }


  /**
   * Vero se il dizionario  stato estratto
   */
  public boolean getDictionariesExtracted() {
        return dictionariesExtracted;
  }

  /**
   * Restituisce l'i-esima feature
   */
  public String getFeature(int i) {
      return (String)((Feature) features.elementAt(i)).getFeature();
  }

  /**
   * Restituisce le feature estratte
   */
  public Vector getFeatures() {
      return features;
  }

  
  /**
   * Restituisce la lunghezza massima di una parola
   */
  public int getMaxLengthTreshold(){
      return maxLengthTreshold;
  }

  /**
   * Restituisce la lunghezza minima di una parola
   */
  public int getMinLengthTreshold(){
      return minLengthTreshold;
  }

  /**
   * Restituisce il numero di feature
   */
  public int getTotalFeatures() {
      return totalFeatures;
  }

  

  /**
   * Restituisce il dizionario della categoria <i>category</i>
   * estratto dal metodo <b>MaxTF-PF<sup>2</sup>-ICF</b>. 
   */
  public Enumeration listDictionary(String category) {
  Vector outputDictionary = new Vector();
  int i = cgm.categoryIndex(category);
  Vector categoryDictionary = (Vector) categoryDictionaries.elementAt(i);

  WeightedWord ww;
  String itemListDictionary;
  String spazi="                     ";
  String word;
  if (!((Vector) categoryDictionaries.elementAt(i)).isEmpty()) {
   for (int j=0; j<categoryDictionary.size(); j++) {
    ww = (WeightedWord) categoryDictionary.elementAt(j);
    Integer weight  = new Integer(ww.getWeight());
    word=ww.getWord();
    if (word.length()>15) {word=word.substring(1,15);}
    itemListDictionary=word+
             spazi.substring(1,15-word.length())+
             (weight.floatValue()/(float)(10000));
    outputDictionary.addElement(itemListDictionary);
   }
  }  
  return outputDictionary.elements(); 
  }

  /**
   * Restituisce il dizionario di <i>category</i> estratto dai
   * metodi <b>Mutual Information</b> e <b>Odds Ratio</b>
   */
  public Enumeration listDictionaryOM(String category){
	 Vector outputDictionary = new Vector();
	 int i = cgm.categoryIndex(category);
	 System.out.println();
	 System.out.print("categoria:"+category);
	 Vector categoryDictionary = (Vector) dictionariesOfCategories[i];
	 DoubleWeightedWord ww;
	 String itemListDictionary;
	 String spazi = "                              ";
    String word;
	 if (!categoryDictionary.isEmpty()){
	   System.out.println("    -size: "+ categoryDictionary.size());
	   for (int j=0;j<categoryDictionary.size();j++){
		  ww = (DoubleWeightedWord) categoryDictionary.elementAt(j);
		  Double weight = new Double(ww.getWeight());
 	     word = ww.getWord();
		  System.out.println(word);
		  if (word.length()>20){
		    word.substring(1,20);
		  }
		  itemListDictionary = word + spazi.substring(1,20-word.length())+ 
		     							   (weight.doubleValue());
		  outputDictionary.addElement(itemListDictionary);
	   }
	  }
	  return outputDictionary.elements();
  }

  /**
   * Elenca le feature
   */
  public Enumeration listFeatures() {
  	Vector outputDictionary = new Vector();
  	String word;
  
  	System.out.println("Features Size:" +features.size());
  	for (int j=0; j<features.size(); j++) {
    		word = ((Feature) features.elementAt(j)).getFeature();
    		//System.out.println("Feature Loaded:"+word);
    		outputDictionary.addElement(word);
  	}  
  	return outputDictionary.elements(); 
  }

  /*
   * Ordina il dizionario
   */
  private Vector[] sortDictionaries(Hashtable[] V){
    Vector sortedCategoryDictionaries[] = new Vector[cgm.totalCategories()];
	 for(int i=0;i<cgm.totalCategories();i++){
 	    sortedCategoryDictionaries[i] = new Vector();
 	 }
	 for(int i=0;i<cgm.totalCategories();i++){
	   Vector sortedCategoryDictionary = new Vector();
	   Hashtable categoryDictionary = (Hashtable) V[i];
		Enumeration e = categoryDictionary.keys();
		while (e.hasMoreElements()){
		  String word = (String)e.nextElement();
		  Double weight = (Double) categoryDictionary.get(word);
		  DoubleWeightedWord dw = new DoubleWeightedWord (word,weight.doubleValue());
		  int j = 0;
		  boolean fine = false;
		  while ((j<sortedCategoryDictionary.size())&&(!fine)){
		    DoubleWeightedWord dw2 = (DoubleWeightedWord)sortedCategoryDictionary.elementAt(j);
 			 if (dw.getWeight()<dw2.getWeight()) j++; else fine = true;
		  }
		  sortedCategoryDictionary.insertElementAt(dw,j);
		}
		if (sortedCategoryDictionary.size()>categoryDictionaryMaxSize){
		    sortedCategoryDictionary.setSize(categoryDictionaryMaxSize);
		}
		sortedCategoryDictionaries[i] = sortedCategoryDictionary;
		} 
	 return sortedCategoryDictionaries;
  }


  /**
   * Estrae le feature con il metodo della <b>Average Mutual Information</b>.
   */
 public void extractFeaturesMI2(Frame progressWindow) throws SQLException, OperationException  {

	int totCat = cgm.totalCategories();
	int totPages = trpm.getNumber();
System.out.println("numero di pagine e categorie: " + totPages + "  " + totCat);
	db.initAMI(utente);
System.out.println("Inizializzazione effettuata");		
	Hashtable temp[] = new Hashtable[totCat];
	for (int i=0; i<totCat; i++) {
		String categoria = cgm.categoryName(i);
		int numDocPerCat = db.getTrainingPages(utente, categoria).size();
System.out.println("Numero documenti per categoria: " + numDocPerCat);
            Hashtable temp2 = db.extractAMI(utente, categoria, totPages, numDocPerCat);
System.out.println("Vocabolario di categoria " + categoria + " estratto");
		temp[i] = (temp2);
	}
	features.removeAllElements();
	Vector[] v = sortDictionaries(temp);
	for (int i=0; i<v.length; i++) {
		Vector c = (Vector)v[i];
		for (int j=0; j<c.size(); j++) {
			DoubleWeightedWord dw = (DoubleWeightedWord)c.elementAt(j);
			Feature w = new Feature( dw.getWord(), cgm.categoryName(i), dw.getWeight() );
			features.addElement( w );
		}
	}
	db.commitAMI();
	db.putFeature(utente, features, "ef2", db.AMI);
      features.removeAllElements();
      //Ricarica le feature in ordine alfabetico
      features = db.getFeature(utente);
      totalFeatures = features.size();
System.out.println(totalFeatures);
      featuresExtracted = true;
      System.out.println("Features extracted");
	if (progressWindow != null)
		progressWindow.hide();
      

}
  

}
