package uniba.core;
/*

   Porter stemmer in Java. The original paper is in

       Porter, 1980, An algorithm for suffix stripping, Program, Vol. 14,
       no. 3, pp 130-137,

   See also http://www.muscat.com/~martin/stem.html

   Bug 1 (reported by Gonzalo Parra 16/10/99) fixed as marked below.
   (the words 'aed', 'eed', 'oed' leave k at 'a' for step 3, and b[k-1]
   is then out outside the bounds of b.

   Release 2

*/

import java.io.*;

/**
 * Implementazione dell'algoritmo di stemming di Porter.
 */
public class StemmerPorter
{   public char[] b;
    private int i,    /* offset into b */
                j, k, k0;
    private static final int INC = 50;
                      /* unit of size whereby b is increased */
    public StemmerPorter()
    {  b = new char[INC];
       i = 0;
    }

    public void reset() { i = 0; }

    public void add(char ch)
    {  if (i == b.length)
       {  char[] new_b = new char[b.length+INC];
          for (int c = 0; c < b.length; c++) new_b[c] = b[c];
          b = new_b;
       }
       b[i++] = ch;
    }

//-------------------------Segue una mia modifica-------------------------

    public synchronized void toChar(String w) {	
    if (w.length()>INC) {b = new char[b.length+INC];}		
    b=w.toCharArray();
    i=w.length();	
    }

//----------------------------Fine toCahrArray-----------------------------

    public String toString() { return new String(b,0,i); }

    /* cons(i) is true <=> b[i] is a consonant. */

    private boolean cons(int i)
    {  switch (b[i])
       {  case 'a': case 'e': case 'i': case 'o': case 'u': return false;
          case 'y': return (i==k0) ? true : !cons(i-1);
          default: return true;
       }
    }

    /* m() measures the number of consonant sequences between k0 and j. if c is
       a consonant sequence and v a vowel sequence, and <..> indicates arbitrary
       presence,

          <c><v>       gives 0
          <c>vc<v>     gives 1
          <c>vcvc<v>   gives 2
          <c>vcvcvc<v> gives 3
          ....
    */

    private int m()
    {  int n = 0;
       int i = k0;
       while(true)
       {  if (i > j) return n;
          if (! cons(i)) break; i++;
       }
       i++;
       while(true)
       {  while(true)
          {  if (i > j) return n;
                if (cons(i)) break;
                i++;
          }
          i++;
          n++;
          while(true)
          {  if (i > j) return n;
             if (! cons(i)) break;
             i++;
          }
          i++;
        }
    }

    /* vowelinstem() is true <=> k0,...j contains a vowel */

    private boolean vowelinstem()
    {  int i; for (i = k0; i <= j; i++) if (! cons(i)) return true;
       return false;
    }

    /* doublec(j) is true <=> j,(j-1) contain a double consonant. */

    private boolean doublec(int j)
    {  if (j < k0+1) return false;
       if (b[j] != b[j-1]) return false;
       return cons(j);
    }

    /* cvc(i) is true <=> i-2,i-1,i has the form consonant - vowel - consonant
       and also if the second c is not w,x or y. this is used when trying to
       restore an e at the end of a short word. e.g.

          cav(e), lov(e), hop(e), crim(e), but
          snow, box, tray.

    */

    private boolean cvc(int i)
    {  if (i < k0+2 || !cons(i) || cons(i-1) || !cons(i-2)) return false;
       {  int ch = b[i];
          if (ch == 'w' || ch == 'x' || ch == 'y') return false;
       }
       return true;
    }

    private boolean ends(String s)
    {  int l = s.length();
       int o = k-l+1;
       if (o < k0) return false;
       for (int i = 0; i < l; i++) if (b[o+i] != s.charAt(i)) return false;
       j = k-l;
       return true;
    }

    /* setto(s) sets (j+1),...k to the characters in the string s, readjusting
       k. */

    void setto(String s)
    {  int l = s.length();
       int o = j+1;
       for (int i = 0; i < l; i++) b[o+i] = s.charAt(i);
       k = j+l;
    }

    /* r(s) is used further down. */

    void r(String s) { if (m() > 0) setto(s); }

    /* step1() gets rid of plurals and -ed or -ing. e.g.

           caresses  ->  caress
           ponies    ->  poni
           ties      ->  ti
           caress    ->  caress
           cats      ->  cat

           feed      ->  feed
           agreed    ->  agree
           disabled  ->  disable

           matting   ->  mat
           mating    ->  mate
           meeting   ->  meet
           milling   ->  mill
           messing   ->  mess

           meetings  ->  meet

    */

    private void step1()
    {  if (b[k] == 's')
       {  if (ends("sses")) k -= 2; else
          if (ends("ies")) setto("i"); else
          if (b[k-1] != 's') k--;
       }
       if (ends("eed")) { if (m() > 0) k--; } else
       if ((ends("ed") || ends("ing")) && vowelinstem())
       {  k = j;
          if (ends("at")) setto("ate"); else
          if (ends("bl")) setto("ble"); else
          if (ends("iz")) setto("ize"); else
          if (doublec(k))
          {  k--;
             {  int ch = b[k];
                if (ch == 'l' || ch == 's' || ch == 'z') k++;
             }
          }
          else if (m() == 1 && cvc(k)) setto("e");
      }
    }

    /* step2() turns terminal y to i when there is another vowel in the stem. */

    private void step2() { if (ends("y") && vowelinstem()) b[k] = 'i'; }

    /* step3() maps double suffices to single ones. so -ization ( = -ize plus
       -ation) maps to -ize etc. note that the string before the suffix must give
       m() > 0. */

    private void step3() { if (k == k0) return; /* for bug 1 */ switch (b[k-1])
    {
        case 'a': if (ends("ational")) { r("ate"); break; }
                  if (ends("tional")) { r("tion"); break; }
                  break;
        case 'c': if (ends("enci")) { r("ence"); break; }
                  if (ends("anci")) { r("ance"); break; }
                  break;
        case 'e': if (ends("izer")) { r("ize"); break; }
                  break;
        case 'l': if (ends("bli")) { r("ble"); break; }
                  if (ends("alli")) { r("al"); break; }
                  if (ends("entli")) { r("ent"); break; }
                  if (ends("eli")) { r("e"); break; }
                  if (ends("ousli")) { r("ous"); break; }
                  break;
        case 'o': if (ends("ization")) { r("ize"); break; }
                  if (ends("ation")) { r("ate"); break; }
                  if (ends("ator")) { r("ate"); break; }
                  break;
        case 's': if (ends("alism")) { r("al"); break; }
                  if (ends("iveness")) { r("ive"); break; }
                  if (ends("fulness")) { r("ful"); break; }
                  if (ends("ousness")) { r("ous"); break; }
                  break;
        case 't': if (ends("aliti")) { r("al"); break; }
                  if (ends("iviti")) { r("ive"); break; }
                  if (ends("biliti")) { r("ble"); break; }
                  break;
        case 'g': if (ends("logi")) { r("log"); break; }
    } }

    /* step4() deals with -ic-, -full, -ness etc. similar strategy to step3. */

    private void step4() { switch (b[k])
    {
        case 'e': if (ends("icate")) { r("ic"); break; }
                  if (ends("ative")) { r(""); break; }
                  if (ends("alize")) { r("al"); break; }
                  break;
        case 'i': if (ends("iciti")) { r("ic"); break; }
                  break;
        case 'l': if (ends("ical")) { r("ic"); break; }
                  if (ends("ful")) { r(""); break; }
                  break;
        case 's': if (ends("ness")) { r(""); break; }
                  break;
    } }

    /* step5() takes off -ant, -ence etc., in context <c>vcvc<v>. */

    private void step5()
    {   if (k == k0) return; /* for bug 1 */ switch (b[k-1])
        {  case 'a': if (ends("al")) break; return;
           case 'c': if (ends("ance")) break;
                     if (ends("ence")) break; return;
           case 'e': if (ends("er")) break; return;
           case 'i': if (ends("ic")) break; return;
           case 'l': if (ends("able")) break;
                     if (ends("ible")) break; return;
           case 'n': if (ends("ant")) break;
                     if (ends("ement")) break;
                     if (ends("ment")) break;
                     /* element etc. not stripped before the m */
                     if (ends("ent")) break; return;
	     case 'o': if (ends("ion")) {
				if (j>=0) {					
					if (b[j] == 's' || b[j] == 't') break;
				} else {j=k;}
			   }
                     if (ends("ou")) break; return;
                     /* takes care of -ous */
           case 's': if (ends("ism")) break; return;
           case 't': if (ends("ate")) break;
                     if (ends("iti")) break; return;
           case 'u': if (ends("ous")) break; return;
           case 'v': if (ends("ive")) break; return;
           case 'z': if (ends("ize")) break; return;
           default: return;
        }
        if (m() > 1) k = j;
    }

    /* step6() removes a final -e if m() > 1. */

    private void step6()
    {  j = k;
       if (b[k] == 'e')
       {  int a = m();
          if (a > 1 || a == 1 && !cvc(k-1)) k--;
       }
       if (b[k] == 'l' && doublec(k) && m() > 1) k--;
    }
    public synchronized void stem(int i0)
    {  k = i - 1; k0 = i0;
       if (k > k0+1) { step1(); step2(); step3(); step4(); step5(); step6(); }
       i = k+1;
    }
}

